<?php

namespace DeubertIt\Debug\Command;

use DeubertIt\Debug\Util\DockerComposeUtil;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class DockerCompose extends Command
{
    const PORTS_OK_MESSAGE = "No exported ports in base docker-compose.yml";
    const PORTS_ERROR_MESSAGE = "Exported ports found in docker-compose.yml. This will not work in CI or other managed environments where more than one instance should be able to run at the same time. If they are meant for local development, please move them to docker-compose.override.yml";
    const CONTAINER_NAME_OK_MESSAGE = "No container names found in base docker-compose.yml";
    const CONTAINER_NAME_ERROR_MESSAGE = "Container names found in docker-compose.yml. This will not work in CI or other managed environments where more than one instance should be able to run at the same time. If they are meant for local development, please move them to docker-compose.override.yml";

    /**
     * @var DockerComposeUtil
     */
    protected $composeUtil;

    /**
     * @var SymfonyStyle
     */
    private $io;

    public function __construct(string $name, DockerComposeUtil $composeUtil)
    {
        parent::__construct($name);
        $this->composeUtil = $composeUtil;
    }

    protected function configure()
    {
        $this
            ->addOption('config', null, InputOption::VALUE_REQUIRED, 'Path to docker-compose.yml')
            ->setDescription('Check docker-compose configuration')
            ->setHelp('Print errors / warnings if any issue is found.')
        ;
    }

    public function execute(InputInterface $input, OutputInterface $output)
    {
        $path = $input->getOption("config") ?? getcwd() . '/docker-compose.yml';
        $this->io = new SymfonyStyle($input, $output);

        $result = $this->composeUtil->parse($path);
        $services = $result['services'];

        $code = 0;

        $portsFound = array_filter($services, function ($service) {
            return !empty($service['ports']);
        });

        $containerNameFound = array_filter($services, function ($service) {
            return !empty($service['container_name']);
        });

        if (!empty($portsFound)) {
            $this->io->error(self::PORTS_ERROR_MESSAGE);
            $code = 2;
        } else {
            $this->io->success(self::PORTS_OK_MESSAGE);
        }

        if (!empty($containerNameFound)) {
            $this->io->error(self::CONTAINER_NAME_ERROR_MESSAGE);
            $code = 2;
        } else {
            $this->io->success(self::CONTAINER_NAME_OK_MESSAGE);
        }

        return $code;
    }
}
