<?php
namespace DeubertIt\Eol\Tests\Command;

use DeubertIt\Debug\Command\DockerCompose;
use DeubertIt\Debug\Command\Eol;
use DeubertIt\Debug\Util\DockerComposeUtil;
use PHPUnit\Framework\TestCase;
use Symfony\Component\Console\Application;
use Symfony\Component\Console\Tester\CommandTester;

class DockerComposeTest extends TestCase
{
    /** @var DockerComposeUtil|mixed */
    private $composeUtilMock;

    /** @var CommandTester */
    private $commandTester;

    protected function setUp(): void
    {
        $this->composeUtilMock = $this->getMockBuilder(DockerComposeUtil::class)
            ->disableOriginalConstructor()
            ->getMock();

        $application = new Application();
        $application->add(new DockerCompose('_', $this->composeUtilMock));
        $command = $application->find('_');
        $this->commandTester = new CommandTester($command);
    }

    protected function tearDown(): void
    {
        $this->composeUtilMock = null;
        $this->commandTester = null;
    }

    public function testErrorsCompose(): void
    {
        $this->composeUtilMock
            ->method("parse")
            ->willReturn([
                "services" => [
                    "service1" => [
                        "ports" => ["3000"]
                    ],
                    "service2" => [
                        "container_name" => "test"
                    ]
                ]
            ]);

        $this->commandTester->execute([]);

        $this->assertStringContainsString("Exported ports found in docker-compose.yml", $this->commandTester->getDisplay());
        $this->assertStringContainsString("Container names found in docker-compose.yml", $this->commandTester->getDisplay());
    }

    public function testOkCompose(): void
    {
        $this->composeUtilMock
            ->method("parse")
            ->willReturn([
                "services" => [
                    "service1" => [
                        "build" => "./app",
                        "volumes" => [
                            "volume1"
                        ]
                    ],
                    "service2" => [
                        "build" => "./app2",
                        "volumes" => [
                            "volume1"
                        ]
                    ]
                ]
            ]);

        $this->commandTester->execute([]);

        $this->assertStringContainsString(DockerCompose::CONTAINER_NAME_OK_MESSAGE, $this->commandTester->getDisplay());
        $this->assertStringContainsString(DockerCompose::PORTS_OK_MESSAGE, $this->commandTester->getDisplay());
    }

    public function testSingleErrorCompose(): void
    {
        $this->composeUtilMock
            ->method("parse")
            ->willReturn([
                "services" => [
                    "service1" => [
                        "build" => "./app",
                        "volumes" => [
                            "volume1"
                        ]
                    ],
                    "service2" => [
                        "build" => "./app2",
                        "ports" => ["4000"],
                        "volumes" => [
                            "volume1"
                        ]
                    ]
                ]
            ]);

        $this->commandTester->execute([]);

        $this->assertStringContainsString(DockerCompose::CONTAINER_NAME_OK_MESSAGE, $this->commandTester->getDisplay());
        $this->assertStringContainsString("Exported ports found in docker-compose.yml", $this->commandTester->getDisplay());
    }

}