<?php
namespace DeubertIt\Eol\Tests\Command;

use DeubertIt\Debug\Util\ConfigUtil;
use DeubertIt\Debug\Command\Eol;
use DeubertIt\Debug\Util\VersionParserUtil;
use PHPUnit\Framework\TestCase;
use Symfony\Component\Console\Application;
use Symfony\Component\Console\Tester\CommandTester;

class EolTest extends TestCase
{
    /** @var ConfigUtil|mixed */
    private $composerUtilMock;

    /** @var VersionParserUtil|mixed */
    private $versionParserUtilMock;

    /** @var CommandTester */
    private $commandTester;

    private $testPhpVersions = [
        "7.0" => "eol",
        "7.1" => "eol",
        "7.2" => "security",
        "7.3" => "stable"
    ];

    private $testSymfonyVersions = ['3.4', '4.4', '5.0'];

    protected function setUp(): void
    {
        $this->composerUtilMock = $this->getMockBuilder(ConfigUtil::class)
            ->disableOriginalConstructor()
            ->getMock();

        $this->versionParserUtilMock = $this->getMockBuilder(VersionParserUtil::class)
            ->disableOriginalConstructor()
            ->getMock();

        $application = new Application();
        $application->add(new Eol('_', $this->composerUtilMock, $this->versionParserUtilMock));
        $command = $application->find('_');
        $this->commandTester = new CommandTester($command);
    }

    protected function tearDown(): void
    {
        $this->versionParserUtilMock = null;
        $this->composerUtilMock = null;
        $this->commandTester = null;
    }

    public function testSymfonyVersions()
    {
        $packages = [
            [ "name" => "symfony/http-kernel", "version" => "3.4.x-dev", "_supported" => true ],
            [ "name" => "symfony/http-kernel", "version" => "3.1.x-dev", "_supported" => false ],
            [ "name" => "symfony/http-kernel", "version" => "4.1.x-dev", "_supported" => false ],
            [ "name" => "symfony/http-kernel", "version" => "4.4.x-dev", "_supported" => true ],
        ];

        foreach ($packages as $key => $package) {
            $this->setUp();

            $this->versionParserUtilMock
                ->method("parsePhpVersions")
                ->willReturn($this->testPhpVersions);

            $this->versionParserUtilMock
                ->method("parseSymfonySupportedVersions")
                ->willReturn($this->testSymfonyVersions);

            $this->composerUtilMock
                ->method("getComposerLockJson")
                ->willReturn([
                    "packages" => [$package],
                    "platform" => [ "php" => "7.3.5" ]
                ]);

            $this->commandTester->execute([]);
            $notSupportedString = "Symfony version (".$package['version'].") is not supported";

            if ($package['_supported']) {
                $this->assertStringContainsString(Eol::SYMFONY_OK_MESSAGE, $this->commandTester->getDisplay());
            } else {
                $this->assertStringContainsString($notSupportedString, $this->commandTester->getDisplay());
            }

            $this->tearDown();
        }
    }
}