<?php
namespace DeubertIt\Debug\Util;

use Symfony\Component\Config\FileLocator;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Loader\XmlFileLoader;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;

class ConfigUtil
{
    /**
     * @var ParameterBagInterface
     */
    private $params;

    public function __construct(?ParameterBagInterface $params)
    {
        $this->params = $params;
    }

    public function getAllParams()
    {
        return $this->params->all();
    }

    public function getKernelDir() {
        if ($this->params) {
            return $this->params->get('kernel.project_dir');
        }

        return getcwd();
    }

    public function getComposerLockJson($projectDir = null)
    {
        $content = file_get_contents(($projectDir ?? $this->getKernelDir()) . '/composer.lock');
        return json_decode($content, true);
    }

    public function getComposerJson($projectDir = null)
    {
        $content = file_get_contents(($projectDir ?? $this->getKernelDir()) . '/composer.json');
        return json_decode($content, true);
    }

    public function getComposerExtra()
    {
        return $this->getComposerJson()['extra'];
    }

    public function getContainerConfig()
    {
        $container = new ContainerBuilder();

        if (!$this->params) {
            return [];
        }

        $loader = new XmlFileLoader($container, new FileLocator());
        $loader->load($this->params->get("debug.container.dump"));

        return $container;
    }

    public function getContainerClass($name) {
        $config = $this->getContainerConfig();

        if (!$config->hasDefinition($name)) {
            return null;
        }

        return $config->getDefinition($name)->getClass();
    }

    public function getContainerAlias($name) {
        $config = $this->getContainerConfig();

        if (!$config->hasAlias($name)) {
            return null;
        }

        return $config->getAlias($name);
    }

    public function getSharedFolders()
    {
        $extra = $this->getComposerExtra();

        if (empty($extra['folders'])) {
            return [];
        }

        $folders = array_filter($extra['folders'], function ($v, $k) {
            return $v['shared'] == true;
        }, ARRAY_FILTER_USE_BOTH);

        return $this->normalizePaths(array_keys($folders));
    }

    public function normalizePath($path)
    {
        return (substr($path, 0, 1) == '/') ? $path : $this->params->get('kernel.project_dir') . '/' . $path;
    }

    public function normalizePaths($paths)
    {
        return array_map(function ($v) {
            return $this->normalizePath($v);
        }, array_keys($paths));
    }

}